#pragma once

#include <thd_mapping/gdm_algorithm.h>
#include <thd_mapping/oct_runner.hpp>
#include <nav_msgs/OccupancyGrid.h>
#include <ros/ros.h>
#include <boost/shared_ptr.hpp>
#include <thd_mapping/index.hpp>
namespace thd_mapping {
    
    THD_MAPPING_CLASS_FORWARD(GaussianRandomFieldAlgorithm)

    class GaussianRandomFieldAlgorithm : public GDMAlgorithm
    {
    public:
        GaussianRandomFieldAlgorithm(GridMap && grid);

        void addDataPoint(const DataPoint& dataPoint);
        void addDataPoints(const std::vector<DataPoint>& dataPoints);
        void occupancyCallback(const nav_msgs::OccupancyGrid::ConstPtr&);
        void setOccupancyMap(thd_mapping::GridMap);
        GridMap getVarianceMap() const {
            if (_varianceMap._cov)
            {
                int maxrows, maxcols;
                std::vector<float> _new;
                _new.reserve(_varianceMap.dim.y*_varianceMap.dim.x);
                maxrows = maxcols =_varianceMap.dim.y*_varianceMap.dim.x;
                thd_mapping::Index index(maxcols, maxrows);
                for (octave_idx_type row = 0; row < maxrows; row++)
                {
                    for (octave_idx_type col = 0; col < maxcols; col++)
                    {
                        if (row == col)
                            _new.push_back(_varianceMap.grid[index(col,row)]);
                    }
                }
                return thd_mapping::GridMap(std::move(_new), _varianceMap.dim, _varianceMap.origin, _varianceMap.resolution);
            }
            else
            {
                return _varianceMap;
            }

        }
        void setDelay(int delay) {
            _delay = delay;
        }

        const GridMap& getCovarianceMap() const {
            return _varianceMap;
        }
    protected:
        thd_mapping::GridMap _varianceMap;
        thd_mapping::GridMap _occupancyMap;
    private:
        void run();
        
        ros::Subscriber _occupancySub;
        int _delay;
        int _previous;
        std::map<int, DataPoint> _readings;

        
    };

}
