#pragma once
#include <vector>
#include <random>
#include <stdint.h>
#include <glm/glm.hpp>
#include <glm/ext.hpp>
#include <oct.h>
#include <ov-re-mat.h>

#include <thd_mapping/data_types.h>

#include <nav_msgs/OccupancyGrid.h>

#include <thd_mapping/MeasurementMap.h>

namespace thd_mapping {

struct GridMap
{
    GridMap() {}
    GridMap(std::vector<float> && grid, glm::ivec2 dim, Position origin, float resolution);
    std::vector<float> grid;
    glm::ivec2 dim;
    Position origin;
    float resolution;

    Indices toGridIndices(const Position& inPos);
    Position toPosition(const Indices& inIndices);
    void operator=(Matrix  &&lhs);

    inline float& operator () (int x, int y) {
        if (!_cov)
        {
            return grid[x + y * dim.x]; 
        }
        return grid[x + y * dim.x * dim.y];
    }

    inline float operator () (int x, int y) const
    {
        if (!_cov)
        {
            return grid[x + y * dim.x]; 
        }
        return grid[x + y * dim.x * dim.y];
    }

    nav_msgs::OccupancyGrid toOccupancyGrid() const;
    MeasurementMap toMeasurementMap() const;

    bool _cov;
};

}
