#pragma once
#include <map>
#include <string>
#include <octave/oct.h>
#include <octave/octave.h>
#include <octave/parse.h>
#include <oct-map.h>

#include <thd_mapping/grid_map.h>
namespace thd_mapping{
    /// This class wraps the calling of Octave using our agreed calling conventions.

    /// The runner can run in two modes. If the runner is constructed with pAuto==1, the object will open and close the IPC automatically, at its own discretion. (This is also the only mode that works at the moment.)
    /// If pAuto==2, the class will attempt to maintain an open connection but leaves no guarantees on state.
    /// Otherwise, the caller will have to manage the IPC using .start() and .end()
    class OctRunner
    {
    public:
        /// Sets the parameters object ahead of time.
        /// @param [in] pParams key,value pair of values. For example, if the key is xpos, then this can be accessed in Octave code using params.xpos
        void setParams(std::map<std::string, double> pParams);
        
        /// Executes the function as defined in the constructor.
        /// @param [in] pPrior current map 
        /// @param [in] pValue the reading value
        /// @param [in] pX the x position 
        /// @param [in] pY the y position
        /// @param [in] pExtraMap1 first extra map parameter (variance)
        /// @param [in] pExtraMap2 second extra map parameter (occupancy)
        /// @param [out] -1 if error, otherwise number of results
        int call(thd_mapping::GridMap& pPrior, const float pValue, int pX, int pY, thd_mapping::GridMap  * pExtraMap1, thd_mapping::GridMap  *  pExtraMap2);
        int call(thd_mapping::GridMap& pPrior, const std::vector<float>& pValue, std::vector<int> & pX, std::vector<int> & pY, thd_mapping::GridMap  * pExtraMap1, thd_mapping::GridMap  *  pExtraMap2);
        int call(thd_mapping::GridMap& pVariance, thd_mapping::GridMap& pOccupancy, thd_mapping::Position, glm::vec2 dim); 
        /// Executes the function as defined in the constructor.
        /// @param [out] -1 if error, otherwise number of results
        int callInit();

        int prepare(std::string pName, std::string pDir);

        int done(std::string pDirInverse);

        /// Returns the number of results
        int countResults();
        
        template<typename T>
        T getResult(int i) 
        {
            if (i >= (_results.length()))
            {
                throw new std::range_error(std::string("Out of bounds getting item ") + std::to_string(i) + " from OctRunner");
            }
            return _results(i).matrix_value();
        }

        static  OctRunner & getInstance()  {
            if (_instance == nullptr)
                _instance = new OctRunner();
            return *_instance;
        }
        
    private:
        OctRunner();
        ~OctRunner();

        octave_map _params;
        std::string _name;
        std::string _dir;
        bool _auto;
        octave_value_list _results;
        int _state;
        static OctRunner * _instance;
        enum STATES {
            PREPARE, READY, POST
        };
    };
}
