#pragma once
#include <thd_mapping/data_types.h>
#include <string>

namespace thd_mapping {

/**
 * @brief The SensorNode class
 * The interface for defining further sensor nodes.
 */
class SensorNode {

public:    
  /**
   * @brief Get the position of this sensor in the `map` frame.
   * @return The position, returned as a thd_mapping::Position object.
   */
  virtual Position getPosition() const = 0;

  /**
   * @brief Get the reading from this sensor.
   * The readings are in the order: temperature, humidity and dust.
   * @return The reading, returned as a thd_mapping::Reading object.
   */
  virtual Reading getReading() const = 0;

  /**
   * @brief Get the name of this sensor node.
   * @return The name associated with this sensor node as a std::string.
   */
  virtual std::string getName() const = 0;

  /**
   * @brief Virtual destructor.
   */
  virtual ~SensorNode() {}

};

}
