#pragma once
#include <thd_mapping/sensor_node.h>
#include <string>
#include <aqs_node/THD.h>
#include <ros/ros.h>
#include <boost/function.hpp>

namespace thd_mapping {

/**
 * @brief The StaticNode class
 * Defines a static node. Derives from SensorNode
 */
class StaticNode : public SensorNode {
public:
  /**
   * @brief Create a StaticNode object from the name and position.
   * @param pName The name of this static node.
   * @param pPos The position of this static node.
   */
  StaticNode(const std::string& pName, const Position& pPos);

  /**
   * @brief StaticNode
   * @param pName The name of this static node.
   * @param pPos The position of this static node.
   * @param callback_fn The callback to be called when new data is available.
   */
  StaticNode(const std::string& pName, const Position& pPos, boost::function<void(const SensorNode&)> callback_fn);

  /**
   * @brief Overridden function. Get the name of this static node.
   * @return Return the name.
   */
  virtual std::string getName() const { return mName; }

  /**
   * @brief Overridden function. Get the position of this static node.
   * @return The position.
   */
  virtual Position getPosition() const;

  /**
   * @brief Overridden function. Get the reading of this static node.
   * @return
   */
  virtual Reading getReading() const;

private:
  /**
   * @brief The reading of this static node.
   */
  Reading mReading;

  /**
   * @brief The position of this static node.
   */
  Position mPosition;

  /**
   * @brief The name of this static node.
   */
  const std::string mName;

  /**
   * @brief thdCallback The callback function.
   * @param thd_reading A const smart pointer to a const object of type aqs_node::THD
   */
  void thdCallback(const aqs_node::THD::ConstPtr& thd_reading);

  /**
   * @brief mSubscriber A ROS subscriber to listen to the THD messages.
   */
  ros::Subscriber mSubscriber;

  /**
   * @brief mCallback A function pointer to call a user-defined function for async reading.
   */
  boost::function<void(const SensorNode&)> mCallback;
};

}
