#include <thd_mapping/file_util.hpp>

#include <iostream>
#include <sstream>
#include <fstream>
#include <algorithm>
#include <exception>


namespace thd_mapping {

std::vector<thd_mapping::DataPoint> dataPointsFromCSV(std::string filename)
{
  std::fstream input;

  std::istringstream ss;
  std::string token;
  std::string line;
  std::vector<thd_mapping::DataPoint> dataPoints;
  try {
    input = std::fstream(filename, std::fstream::in);
  }
  catch(std::exception& e)
  {
    std::cout << "Error: " << e.what();
    return dataPoints;
  }

  std::getline(input, line); //Headers
  while (std::getline(input, line)) {

    thd_mapping::DataPoint pointUSB[3];
    thd_mapping::DataPoint point5[3];
    thd_mapping::DataPoint point9[3];


    // point5.position = { 11.0f, 5.0f };
    // pointUSB.position = { 10.0f, 10.0f};
    // point9.position = { 9.0f, 15.0f};

    point5[2].position = point5[1].position = point5[0].position = {0.0f, -3.8f };
    pointUSB[2].position = pointUSB[1].position = pointUSB[0].position = { 1.54f, 0.3f};
    point9[2].position = point9[1].position = point9[0].position = { 2.4f, 3.9f};

    auto lineStream = std::istringstream(line);

    std::getline(lineStream, token, ',');

    point5[2].time = point5[1].time = point5[0].time = atol(token.c_str());;
    pointUSB[2].time = pointUSB[1].time = pointUSB[0].time = point5[0].time;
    point9[2].time = point9[1].time = point9[0].time = point5[0].time;

    std::getline(lineStream, token, ',');
    point5[2].node_id = point5[1].node_id = point5[0].node_id = 5;
    std::getline(lineStream, token, ',');
    point5[0].data = atol(token.c_str());
    std::getline(lineStream, token, ',');
    point5[1].data = atol(token.c_str());
    std::getline(lineStream, token, ',');

    point9[2].node_id = point9[1].node_id = point9[0].node_id = 9;
    std::getline(lineStream, token, ',');
    point9[0].data = atol(token.c_str());
    std::getline(lineStream, token, ',');
    point9[1].data = atol(token.c_str());
    std::getline(lineStream, token, ',');

    pointUSB[2].node_id = pointUSB[1].node_id = pointUSB[0].node_id = 9;
    std::getline(lineStream, token, ',');
    pointUSB[0].data= atol(token.c_str());
    std::getline(lineStream, token, ',');
    pointUSB[1].data = atol(token.c_str());
    std::getline(lineStream, token, ',');
    //std::cout << token << time << '\n';

    // dataPoints.push_back(pointUSB[0]);
    // dataPoints.push_back(pointUSB[1]);
    dataPoints.push_back(pointUSB[0]);
    // dataPoints.push_back(point5[0]);
    // dataPoints.push_back(point5[1]);
    dataPoints.push_back(point5[0]);
    // dataPoints.push_back(point9[0]);
    // dataPoints.push_back(point9[1]);
    dataPoints.push_back(point9[0]);
  }

  return dataPoints;
}

void writePPMFromGridMap(const thd_mapping::GridMap& gridMap, std::string outputName)
{
  std::fstream out;
  //out.open("img.ppm", std::ios::binary);

  float maxValue = *std::max_element(gridMap.grid.begin(), gridMap.grid.end());
  float minValue = *std::min_element(gridMap.grid.begin(), gridMap.grid.end());


  // float maxValue = 24;
  // float minValue = 20;

  out.open(outputName, std::fstream::out);
  out << "P2" << "\n"
      << gridMap.dim.x << " "
      << gridMap.dim.y << "\n"
      << 255 << "\n";

  for (int i = 0; i<gridMap.dim.x; i++)
  {
    for (int j = 0; j<gridMap.dim.y; j++)
    {
      int value = int((gridMap.grid[i + j*gridMap.dim.x] - minValue) * 255.0f / (maxValue - minValue));
      out << value << " ";
      //out << gridMap.grid[i + j*gridMap.dim.x] * 255.0f << " ";
    }
  }
  out.close();
}
}
