#include <thd_mapping/gaussian_kernel_weight_mapper.h>

namespace thd_mapping {

GaussianKernelWeightMapper::GaussianKernelWeightMapper(GridMap && grid, int kernelSize) :
  GDMAlgorithm(std::move(grid)), mKernelSize(kernelSize)
{
    mKernel = createGaussianKernel(kernelSize);
}

GaussianKernelWeightMapper::~GaussianKernelWeightMapper() {}

void GaussianKernelWeightMapper::addDataPoint(const DataPoint& dataPoint)
{
  // All three maps have same resolution. So this is okay.
  glm::vec2 gridMapPos = this->mGrid.toGridIndices(glm::vec2(dataPoint.position.x,dataPoint.position.y));

  int halfKernelSize = (mKernelSize / 2);
  glm::ivec2 xy;
  for (xy.y = -halfKernelSize; xy.y <= halfKernelSize; xy.y++)
  {
    for (xy.x = -halfKernelSize; xy.x <= halfKernelSize; xy.x++)
    {

      glm::ivec2 gridPos = { gridMapPos.x + xy.x, gridMapPos.y + xy.y };
      if (!(gridPos.x >= 0 && gridPos.x < this->mGrid.dim.x && gridPos.y >= 0 && gridPos.y < this->mGrid.dim.y)) continue;

      const float kernelValue = mKernel[(xy.x + halfKernelSize) + (xy.y + halfKernelSize) * mKernelSize];
      float& gridValue = this->mGrid(gridPos.x, gridPos.y);
      gridValue = gridValue * (1 - kernelValue) + dataPoint.data*kernelValue;

    }
  }

}

void GaussianKernelWeightMapper::addDataPoints(const std::vector<DataPoint>& dataPoints)
{
  for(auto p : dataPoints)
  {
    addDataPoint(p);
  }
}

}

