#include <thd_mapping/grf_algorithm.hpp>
#include <glm/vec2.hpp>
#include <ros/ros.h>
#include <nav_msgs/OccupancyGrid.h>

namespace thd_mapping
{
    GaussianRandomFieldAlgorithm::GaussianRandomFieldAlgorithm(thd_mapping::GridMap && map) : GDMAlgorithm(std::move(map))
    {

        this->_varianceMap = thd_mapping::GridMap(std::vector<float>(pow(mGrid.dim.x*mGrid.dim.y,2), 0), {mGrid.dim.x, mGrid.dim.y}, { 0,0 }, 1.f);
        _varianceMap._cov = true;
        this->_occupancyMap =  thd_mapping::GridMap(std::vector<float>((mGrid.dim.x*mGrid.dim.y), 1), {mGrid.dim.x, mGrid.dim.y}, { 0,0 }, 1.f);
        

//        if (ros::isInitialized()) // helps with testing
//        {
//            ros::NodeHandle nh;
//            _occupancySub = nh.subscribe("/map", 10, &GaussianRandomFieldAlgorithm::occupancyCallback, this);;
//        }
    }

    
    void GaussianRandomFieldAlgorithm::occupancyCallback(const nav_msgs::OccupancyGrid::ConstPtr& msg)
    {
        
        glm::ivec2 lDim = {
            lDim.x = msg->info.width,
            lDim.y = msg->info.height
        };
        
        _occupancyMap = thd_mapping::GridMap(
            std::vector<float>(&msg->data[0], &msg->data[0] + msg->data.size()),
            lDim,
            glm::vec2(msg->info.origin.position.x, msg->info.origin.position.y),
            msg->info.resolution);
    }
    
    void GaussianRandomFieldAlgorithm::setOccupancyMap(thd_mapping::GridMap val){
        _occupancyMap = val;
        
    }
    void GaussianRandomFieldAlgorithm::addDataPoint(const thd_mapping::DataPoint& dataPoint)
    {           
        _readings[dataPoint.node_id] = dataPoint;
        run();
    }

    void GaussianRandomFieldAlgorithm::run(){
        if ((time(NULL) - _previous) < _delay)
        {
            return;
        }
        std::vector<int> x;
        std::vector<int> y;
        std::vector<float> data;
        for (auto pair : _readings)
        {
            glm::vec2 gridMapPos = this->mGrid.toGridIndices(glm::vec2(pair.second.position.x, pair.second.position.y));
            x.push_back(gridMapPos.x);
            y.push_back(gridMapPos.y);
            data.push_back(pair.second.data);
        }

        thd_mapping::OctRunner::getInstance().prepare("randomField", "matlab");
        thd_mapping::OctRunner::getInstance().setParams({
                {"neighbour_sigma", 1.0},
                {"sensor_sigma", 0.1},
                {"time_sigma", 0.1},
                {"error_tolerance", 0.00001},
                {"occupancyThreshold", 2}
            });

        std::cout << "Running now with " << x.size() << " points" << std::endl;
        thd_mapping::OctRunner::getInstance().call(this->mGrid, data,  x, y,  &this->_varianceMap, &this->_occupancyMap);
        mGrid = thd_mapping::OctRunner::getInstance().getResult<Matrix>(0);
               
        _varianceMap = thd_mapping::OctRunner::getInstance().getResult<Matrix>(1);
        thd_mapping::OctRunner::getInstance().done("..");
        _previous = time(NULL);        
    }
    
    void GaussianRandomFieldAlgorithm::addDataPoints(const std::vector<DataPoint>& dataPoints)
    {
      std::cout << "Number of points: " << dataPoints.size() << std::endl ;
      for (auto point : dataPoints)
      {
        _readings[point.node_id] = point;
      }
      run();
    }
        
}
