#include <thd_mapping/grid_map.h>

namespace thd_mapping {

GridMap::GridMap(std::vector<float> && grid, glm::ivec2 dim, Position origin, float resolution) :
  grid(std::move(grid)), dim(dim), origin(origin), resolution(resolution), _cov(false)
{

}

Indices GridMap::toGridIndices(const Position& inPos)
{
  return (inPos - origin) / resolution;
}

Position GridMap::toPosition(const Indices& inIndices)
{
  return (inIndices * resolution) + origin;
}

void GridMap::operator=(Matrix && rhs)
{
  for (octave_idx_type row = 0; row < rhs.rows(); row++)
  {
    for (octave_idx_type col = 0; col < rhs.cols(); col++)
    {
      this->grid[col + rhs.cols() * row] = rhs(row,col);
    }
  }
}


nav_msgs::OccupancyGrid GridMap::toOccupancyGrid() const {

  float maxValue = *std::max_element(this->grid.begin(), this->grid.end());
  float minValue = *std::min_element(this->grid.begin(), this->grid.end());

  nav_msgs::OccupancyGrid mapMsg;
  mapMsg.header.frame_id = "map";

  mapMsg.info.resolution = this->resolution;
  mapMsg.info.width = this->dim.x;
  mapMsg.info.height = this->dim.y;

  mapMsg.info.origin.position.x = this->origin.x;
  mapMsg.info.origin.position.y = this->origin.y;

  mapMsg.info.origin.orientation.w = 1.0;
  mapMsg.info.origin.orientation.x = 0.0;
  mapMsg.info.origin.orientation.y = 0.0;
  mapMsg.info.origin.orientation.z = 0.0;

  mapMsg.data.reserve(this->grid.size());
  for (auto value : this->grid)
  {
    mapMsg.data.push_back(((value - minValue) * 100.0f  / (maxValue - minValue)));
  }
  return mapMsg;
}

MeasurementMap GridMap::toMeasurementMap() const {

  float maxValue = *std::max_element(this->grid.begin(), grid.end());
  float minValue = *std::min_element(grid.begin(), grid.end());


  MeasurementMap measurementMapMsg;
  measurementMapMsg.header.stamp = ros::Time::now();
  measurementMapMsg.header.frame_id = "map";


  measurementMapMsg.info.resolution = resolution;
  measurementMapMsg.info.width = static_cast<uint32_t>(dim.x);
  measurementMapMsg.info.height = static_cast<uint32_t>(dim.y);
  measurementMapMsg.maxValue = maxValue;
  measurementMapMsg.minValue = minValue;
  measurementMapMsg.type = "temp";

  measurementMapMsg.info.origin.position.x = this->origin.x;
  measurementMapMsg.info.origin.position.y = this->origin.y;

  measurementMapMsg.info.origin.orientation.w = 1.0;
  measurementMapMsg.info.origin.orientation.x = 0.0;
  measurementMapMsg.info.origin.orientation.y = 0.0;
  measurementMapMsg.info.origin.orientation.z = 0.0;


  measurementMapMsg.data.reserve(grid.size());
  for (auto value : grid)
  {
    measurementMapMsg.data.push_back(((value - minValue) * 100.0  / (maxValue - minValue)));
  }

  return measurementMapMsg;
}
}
