#include <thd_mapping/kernel_dmv_algorithm.hpp>

namespace thd_mapping {

float N(const Position& x_i, const Position& xk, const float sigma) {
  float something = -glm::distance(static_cast<glm::vec2>(x_i), static_cast<glm::vec2>(xk))*
      glm::distance(static_cast<glm::vec2>(x_i), static_cast<glm::vec2>(xk))/(2*sigma*sigma);
  float return_value = (1/(sigma*sqrt(2*glm::pi<float>())))*(std::exp(something));
  return return_value;
}

KernelDMVAlgorithm::KernelDMVAlgorithm(GridMap && m_grid,
                                       GridMap && v_grid,
                                       float kernel_sigma,
                                       float scaling_sigma,
                                       bool compute_variance_map) :
  GDMAlgorithm(std::move(m_grid)),
  vGrid(std::move(v_grid)),
  kernelSigma(kernel_sigma),
  scalingSigma(scaling_sigma),
  computeVarianceMap(compute_variance_map)
{
  mKernelSize = kernelSigma * 2.0 + 1.0;
}

KernelDMVAlgorithm::~KernelDMVAlgorithm() {}

void KernelDMVAlgorithm::addDataPoint(const DataPoint& dataPoint)
{

}

void KernelDMVAlgorithm::addDataPoints(const std::vector<DataPoint>& dataPoints)
{
  int halfKernelSize = (mKernelSize / 2);
  glm::ivec2 xy;

  for (int i = 0; i < mGrid.dim.x; ++i)
    for (int j = 0; j < mGrid.dim.y; ++j)
    {
      float omega = 0.0;
      float R = 0.0;
      float confidence = 0.0;

      Position pointToUpdate = mGrid.toPosition(Indices(i,j));

      for (auto dataPoint : dataPoints) {
        // If this point is not affected by this reading, move on.
        //if(glm::distance(Indices(i,j), mGrid.toGridIndices((dataPoint.position))) > halfKernelSize) {
          //printf("[OK]: (%f, %f) and (%f, %f)", dataPoint.position.x, dataPoint.position.y, pointToUpdate.x, pointToUpdate.y);
          //continue;
        //}

        const Position& dataPosition = dataPoint.position;

        omega += N(dataPosition, pointToUpdate, this->kernelSigma);
        R += N(dataPosition, pointToUpdate, this->kernelSigma) * dataPoint.data;
      }

      // Update confidence map.
      confidence = 1 - exp(-omega*omega/(scalingSigma*scalingSigma));
      // Update mean map.
      mGrid(i,j) = ( confidence * (R/omega) ) + ( (1 - confidence)*(mGrid(i,j)) );

      // If we are running KernelDM, return here.
      if(!computeVarianceMap) return;

      float tau = 0.0;
      float V = 0.0;
      float v0 = 0.0;

      for (auto dataPoint : dataPoints) {
        // If this point is not affected by this reading, move on.
        if(glm::distance(Indices(i,j), mGrid.toGridIndices((dataPoint.position))) > halfKernelSize) continue;

        const Position& dataPosition = dataPoint.position;

        tau = (dataPoint.data - mGrid(i,j))*(dataPoint.data - mGrid(i,j));
        v0 += tau;
        V += N(dataPosition, pointToUpdate, this->kernelSigma)*tau;
      }

      // Compute variance prior.
      v0 = v0/dataPoints.size();

      // Update the variance map.
      vGrid(i,j) = ( confidence * (V/omega) ) + ( (1 - confidence)*v0);
    }
}

}
