
#include <string>
#include <thd_mapping/sensor_node.h>
#include <thd_mapping/basic_node.h>
#include <thd_mapping/smoothing_node.hpp>
#include <thd_mapping/median_node.hpp>
#include <glm/vec3.hpp>
#include <memory>
#include <boost/tokenizer.hpp>
#include <thd_mapping/file_util.hpp>
#include <thd_mapping/grf_algorithm.hpp>
#include <thd_mapping/oct_runner.hpp>
#include <thd_mapping/max_variance_goal_finder.hpp>
#include <map>
#include <octave/octave.h>
#include <octave/toplev.h>
#include <iostream>
#include <fstream>
#include <thd_mapping/index.hpp>

bool compare (const thd_mapping::GridMap& map, std::string fileName, bool print=false) 
{
    bool equal = true;
    std::ifstream file(fileName);;

    typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
    boost::char_separator<char> sep(",");
    if (print)
        std::cout << "CSV-Data:" << std::endl;
    int maxRows = map._cov ? map.dim.y*map.dim.x : map.dim.y;
    int maxCols = map._cov ? map.dim.y*map.dim.x : map.dim.x;
    std::cout << "Mx, My " << maxCols << ", " << maxRows << std::endl;
    thd_mapping::Index index(maxCols, maxRows);
    for(int row = 0; row < maxRows; ++row)
    {
        std::string line;
        std::getline(file, line);
        if ( !file.good() ) 
            break;
        tokenizer tokens(line, sep);
        int col = 0;

        for (tokenizer::iterator it = tokens.begin(); it != tokens.end(); it++)
        {
            if (row == col)
            {
                double mapVal =  map(col,row), csvVal = strtod(it->c_str(), nullptr);
                equal &= abs(mapVal - csvVal) < 1e-2;
                if (print)
                {
                    std::cout << csvVal;
                    if ( abs(mapVal - csvVal) > 1e-2)
                    {
                        std::cout << "?";
                        std::cout << mapVal << " " << csvVal << std::endl;
                     }
                    if (col < maxCols-1)
                    {
                        std::cout << ", ";
                    } else {
                        std::cout << std::endl;
                    }
                }
            }
            col++;
        }
    }
    if (print)
    {
        std::cout << "\nMapData: " <<std::endl;
        for (int row = 0; row < maxRows; ++row)
        {
            for (int col = 0; col < maxCols; ++col)
            {
                if (row == col)
                {
                double mapVal =  map.grid[index(col, row)];
                
                std::cout << mapVal;
                if (col < maxCols - 1)
                {
                    std::cout << ", ";
                } else {
                    std::cout << std::endl;
                }
                }
            }   
        }
    }
    return equal;
}

bool goaltest(thd_mapping::Position pos, thd_mapping::Position expected, bool large=false)
{
    
    thd_mapping::GridMap grid;
    std::vector<float> data({
            1,   5,  16,   0,  19,  15,
           10,  12,   4,   9,  20,  11,
           10,   4,  10,  13,  14,   8,
            7,  20,   0,  18,  19,  16,
                }), data2({
  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  5,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0, 16,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0, 19,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0, 12,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  9,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 20,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 11,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 10,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 13,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 14,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  8,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  7,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 20,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 18,  0,  0
,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 19,  0
, 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 16
});
    //, {6,4}, {0,0}, 1.f);
    if (large)
    {
        grid = thd_mapping::GridMap(std::move(data2), {6,4}, {0,0}, 1.f);
        grid._cov = true;
    } else {
        grid = thd_mapping::GridMap(std::move(data), {6,4}, {0,0}, 1.f);
    }
    thd_mapping::GridMap occ(std::vector<float>(24, 1), {6,4}, {0,0}, 1.f);
    thd_mapping::MaxVarianceGoalFinder finder(grid, occ);
    finder.setPosition(pos);
    thd_mapping::Position goal = finder.getGoal();

    if (goal == expected)
    {
        std::cout << "Goal finding test passed!" << std::endl;
    }
    else
    {
        std::cout << "Goal test failed: expected (" << expected.x << ", " << expected.y << ") got (" << goal.x << ", " << goal.y << ")" << std::endl;
    }
}


int main(int argc, char *argv[])
{
    glm::ivec2 dim = { 10,10 };
    float initTemp = 22.0f;
    float initHumidity = 30.0f;
    float initDust = 20.0f;

    auto mappingAlgorithm = std::shared_ptr<thd_mapping::GaussianRandomFieldAlgorithm>(
        new thd_mapping::GaussianRandomFieldAlgorithm(
            thd_mapping::GridMap(std::vector<float>(dim.x*dim.y, 0), dim, { 0,0 }, 1.f))
        );
    
    thd_mapping::GridMap occupancyMap(std::vector<float>(dim.x*dim.y, 1), dim, {0,0}, 1.f);
    std::vector<thd_mapping::DataPoint> dps {
        {0,0,33,{1,5}},
        {1,0,30,{8,2}}
    };

    mappingAlgorithm->setOccupancyMap(occupancyMap);
    mappingAlgorithm->setDelay(0);
    for (int i = 0; i < 2; i++)
    {
        bool equal = compare(mappingAlgorithm->getMap(), "matlab/testfile/mean" + std::to_string(i) + ".csv");
        std::cout << "Test result for mean " << std::to_string(i) << ": " << (equal ? "passed" : "failed") << std::endl;
        
        equal = compare(mappingAlgorithm->getCovarianceMap(), "matlab/testfile/variance" + std::to_string(i) + ".csv",true);
        std::cout << "Test result for variance " << std::to_string(i) << ": " << (equal ? "passed" : "failed") << "\n"<< std::endl;
        
        mappingAlgorithm->addDataPoints(dps);
    }
    bool equal = compare(mappingAlgorithm->getMap(), "matlab/testfile/mean2.csv");
    std::cout << "Test result for mean 2: " << (equal ? "passed" : "failed") << std::endl;

    equal &= compare(mappingAlgorithm->getCovarianceMap(), "matlab/testfile/variance2.csv");
    std::cout << "Test result for variance 2: " << (equal ? "passed" : "failed") << std::endl;

    std::cout << "\n\nTesting goal finding" << std::endl;
    goaltest({6,4}, {5,2}, true);
    goaltest({2,1}, {2,4});
    clean_up_and_exit(!equal,1);
    return !equal;
}

