PoseStamed = function () {
    this.header =  {
        seq: 0,
        stamp: 0,
        frame_id: 'map'
    }
    this.pose = new ROSLIB.Pose;
}


Renderer = function (options) {
    var that = this;
    options = options || {};
    var ros = options.ros;
    var canvasId = options.canvasId || "canvas";
    var gridToggle = options.gridToggle || "occupancy-grid-toggle";
    var sensorToggle = options.sensorToggle || "sensors-toggle";

    this.canvas = document.getElementById(canvasId);
    this.context = this.canvas.getContext('2d');
    this.offset = { x: 0, y: 0 };
    this.scale = 10; //maxScale

    this.resolution = 0.1;

    this.grid = options.grids || [];
    this.sensors = options.sensors || [];
    this.largestDim = { width: 0, height: 0 };

    let OccupancyGridToggle = $("#"+gridToggle);
    for (let i = 0; i < this.grid.length; i++) {
        let newCheckbox = $('<li><a href="#"><input type="checkbox" checked="true"/>&nbsp;' + that.grid[i].rosTopic.name + '</a></li>').change(function(e) {
            that.grid[i].grid.hidden = !e.target.checked;
        });
        $(OccupancyGridToggle).append(newCheckbox);

        this.grid[i].rosTopic.subscribe(function (message) {
            subscribeHandle(message, this, that.grid[i]);
        });
    }

    let sensorsToggle = $("#"+sensorToggle);
    for (let i = 0; i < this.sensors.length; i++) {
        let newCheckbox = $('<li><a href="#"><input type="checkbox" checked="true"/>&nbsp;' + that.sensors[i].rosTopic.name + '</a></li>').change(function(e) {
            that.sensors[i].sensor.hidden = !e.target.checked;
        });
        $(sensorsToggle).append(newCheckbox);

        this.sensors[i].rosTopic.subscribe(function (message) {
            PoseStampedSubscribeHandle(message, this, that.sensors[i].sensor);
        });
    }

    this.navGoal = {
        // sensor: new PoseStampedViewer({}),
        message: new PoseStamed(),
        rosTopic: new ROSLIB.Topic({
            ros: ros,
            name: "/move_base_simple/goal",
            messageType: 'geometry_msgs/PoseStamped',
        }),
    };

    // this.sensors.push(this.navGoal);

    function pixelToRosPos(pos)
    {
        return {x:(pos.x - that.offset.x) * that.resolution/that.scale,
                y:-(pos.y - that.offset.y) * that.resolution/that.scale
        }
    }

    function getMousePos(canvas, evt) {
        var rect = canvas.getBoundingClientRect();
        return pixelToRosPos({
            x: evt.clientX - rect.left,
            y: evt.clientY - rect.top
        });
    }

    this.navGoal.rosTopic.advertise();
    this.canvas.addEventListener('click', function (evt) {
        var mousePos = getMousePos(that.canvas, evt);
        that.navGoal.message.pose.position.x = mousePos.x;
        that.navGoal.message.pose.position.y = mousePos.y;
        publish(that.navGoal);
    });

    function PoseStampedSubscribeHandle(message, rosSubcriber, sensor) {
        sensor.message = message;

    }

    function publish(rosPublisher,pos) {
        rosPublisher.rosTopic.publish(rosPublisher.message);
        // rosPublisher.sensor.message = rosPublisher.message;
        // console.log(rosPublisher.message);
    }

    function subscribeHandle(message, rosSubcriber, grid) {

        grid.grid.message = message;

        if(grid.scale)
        {
            grid.scale.maxValue = message.maxValue;
            grid.scale.minValue = message.minValue;
        }

        if (that.largestDim.width < message.info.width) {
            scale = that.canvas.width / message.info.width;
            if (that.scale > scale) that.scale = scale;

            that.largestDim.width = message.info.width;
            that.offset.x = -message.info.origin.position.x / message.info.resolution * scale;
        }
        if (that.largestDim.height < message.info.height) {
            scale = that.canvas.height / message.info.height;
            if (that.scale > scale) that.scale = scale;

            that.largestDim.height = message.info.height;
            that.offset.y = that.canvas.height + message.info.origin.position.y / message.info.resolution * scale;
            that.resolution = message.info.resolution;
        }

        that.redraw(message);
    }


    this.redraw = function () {
        this.context.fillStyle = "black";
        this.context.fillRect(0, 0, this.canvas.width, this.canvas.height);

        for (let i in this.grid) {
            var grid = this.grid[i].grid;
            grid.draw(this.context, this.offset, this.scale);
        }
        for (let i in this.sensors) {
            var sensor = this.sensors[i].sensor;
            sensor.draw(this.context, this.offset, this.scale, this.resolution);
        }

        for (let i in this.grid) {
            var scale = this.grid[i].scale;
            if(scale)
                scale.draw(that.canvas,that.context);
        }
    }


};
Renderer.prototype.__proto__ = EventEmitter2.prototype;
