#!/usr/bin/env python

import socket
import sqlite3
import rospy
import sys
from std_msgs.msg import Float32

##############################
### DEFS
##############################
def add_to_db(time, nname, temp, humidity, occlusion):
  try:
    c.execute('''INSERT INTO measurement (node_id, time, temp, humidity, occlusion)VALUES ((SELECT ID FROM node WHERE name=?), ?, ?, ?, ? );''', (nname, time, temp, humidity, occlusion))
    conn.commit()
  except e:
    print "Error on transcation:", e

def temp_callback(_data):
  global node3_temp
  node3_temp = _data.data
  
def humidity_callback(_data):
  global node3_humidity
  node3_humidity = _data.data
  
def dust_callback(_data):
  global node3_dust
  node3_dust = _data.data

rospy.init_node('data_recorder', anonymous=True)
rospy.Subscriber("/aqs/dust", Float32, dust_callback)
rospy.Subscriber("/aqs/temperature", Float32, temp_callback)
rospy.Subscriber("/aqs/humidity", Float32, humidity_callback)


### CONSTS
UDP_IP = [('192.168.1.120', "N9"), ('192.168.1.105', "N5")]
UDP_PORT = 55005
BUFFER_SIZE = 1024
MESSAGE = "reqData"
rate = rospy.Rate(0.1); # 6 Hz

## Globals
node3_temp = 0.0
node3_humidity = 0.0
node3_dust = 0.0
measurement_cntr = {"N9": 0, "N5": 0, "USB": 0}

## SETUP
dbName = sys.argv[1] if len(sys.argv) > 1 else "sens.db"
print "Recording data into ", dbName, ""
conn = sqlite3.connect(dbName)
c = conn.cursor()

## SETUP TABLES
#c.execute("DROP TABLE IF EXISTS node")
c.execute("CREATE TABLE IF NOT EXISTS node (ID INTEGER PRIMARY KEY AUTOINCREMENT, name BLOB);")
c.execute('''CREATE TABLE IF NOT EXISTS measurement
(ID INTEGER PRIMARY KEY AUTOINCREMENT, 
node_id INTEGER, 
temp REAL, 
humidity REAL, 
occlusion REAL, 
time INTEGER,
FOREIGN KEY(node_id) REFERENCES node(ID))''')

## Create indices
#c.execute("CREATE INDEX IF NOT EXISTS measurementIndex ON measurement(ID)")
c.execute("CREATE INDEX IF NOT EXISTS measurementIndexNode ON measurement(node_id)")
c.execute("CREATE INDEX IF NOT EXISTS measurementIndex ON measurement(time)")
c.execute("SELECT * FROM node WHERE name='N5'")
## Insert node names
c.execute(b"INSERT OR REPLACE INTO node (ID,name) VALUES (1,?)", (b"N5",))
c.execute(b"INSERT OR REPLACE INTO node (ID,name) VALUES (2,'N9')")
c.execute(b"INSERT OR REPLACE INTO node (ID,name) VALUES (3,'USB')")


c.execute("SELECT * FROM node")
print "Nodes added: \n", c.fetchall()



while not rospy.is_shutdown():
  now=int(rospy.Time.now().to_sec())
  for (IP,node) in UDP_IP:
    # Open socket
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(5)
    # Send message
    s.sendto(MESSAGE, (IP, UDP_PORT))
    try:
      # Try to read
      data = s.recv(BUFFER_SIZE)
      data = data.split(";")
      # Avoid "out of bounds"
      if len(data) >= 5:
        add_to_db(now, data[1], float(data[2])/10.0, float(data[3])/10.0, data[4])
        measurement_cntr[data[1]] += 1
    except Exception as e:
      print "Error: ", e
    s.close()
  rate.sleep()
  add_to_db(now,u'USB', node3_temp, node3_humidity, node3_dust)
  measurement_cntr["USB"] += 1

  
